<?php

namespace Rtcl\Controllers\Admin;


use Rtcl\Helpers\Functions;
use Rtcl\Models\Payment;
use Rtcl\Models\RtclEmail;

class PaymentStatus
{

    function __construct() {
        add_action('transition_post_status', [$this, 'transition_post_status'], 10, 3);
    }


    public function transition_post_status($new_status, $old_status, $post) {

        if (rtcl()->post_type_payment !== $post->post_type) {
            return;
        }

        // TODO : need to add some logic

        if ('rtcl-completed' === $new_status && 'rtcl-completed' !== $old_status) {
            $payment = rtcl()->factory->get_order($post->ID);
            if ("membership" != get_post_meta($post->ID, 'payment_type', true)) {
                $listing = rtcl()->factory->get_listing($payment->get_listing_id());
                if (!absint(get_post_meta($post->ID, '_applied', true)) && in_array($listing->get_status(), ['publish', 'rtcl-expired'], true)) {
                    $visible = $payment->pricing->getVisible();
                    $current_date = new \DateTime(current_time('mysql'));
                    $old_expiry_date = get_post_meta($payment->get_listing_id(), 'expiry_date', true);
                    $expiry_date = $old_expiry_date ? new \DateTime(Functions::datetime('mysql', trim($old_expiry_date))) : '';
                    if ($payment->pricing->hasPromotion()) {
                        $needToUpdateExpiredDate = false;
                        if ($payment->pricing->hasFeatured()) {
                            $feature_expiry_date = get_post_meta($payment->get_listing_id(), 'feature_expiry_date', true);
                            $feature_expiry_date = $feature_expiry_date ? new \DateTime(Functions::datetime('mysql', trim($feature_expiry_date))) : '';
                            if ($feature_expiry_date && $current_date < $feature_expiry_date) {
                                $feature_expiry_date->add(new \DateInterval("P{$visible}D"));
                            } else {
                                $feature_expiry_date = new \DateTime(current_time('mysql'));
                                $feature_expiry_date->add(new \DateInterval("P{$visible}D"));
                            }

                            $featureExpDate = $feature_expiry_date->format('Y-m-d H:i:s');
                            update_post_meta($payment->get_listing_id(), 'featured', 1);
                            update_post_meta($payment->get_listing_id(), 'feature_expiry_date', $featureExpDate);

                            if ($expiry_date) {
                                if ($feature_expiry_date > $expiry_date) {
                                    $needToUpdateExpiredDate = true;
                                    $expiry_date = $feature_expiry_date;
                                }
                            } else {
                                $expiry_date = $feature_expiry_date;
                            }

                        }

                        if ($payment->pricing->hasTop()) {
                            $top_expiry_date = get_post_meta($payment->get_listing_id(), '_top_expiry_date', true);
                            $top_expiry_date = $top_expiry_date ? new \DateTime(Functions::datetime('mysql', trim($top_expiry_date))) : '';
                            if ($top_expiry_date && $current_date < $top_expiry_date) {
                                $top_expiry_date->add(new \DateInterval("P{$visible}D"));
                            } else {
                                $top_expiry_date = new \DateTime(current_time('mysql'));
                                $top_expiry_date->add(new \DateInterval("P{$visible}D"));
                            }

                            $topExpDate = $top_expiry_date->format('Y-m-d H:i:s');
                            update_post_meta($payment->get_listing_id(), '_top', 1);
                            update_post_meta($payment->get_listing_id(), '_top_expiry_date', $topExpDate);

                            if ($expiry_date) {
                                if ($top_expiry_date > $expiry_date) {
                                    $needToUpdateExpiredDate = true;
                                    $expiry_date = $top_expiry_date;
                                }
                            } else {
                                $expiry_date = $top_expiry_date;
                            }

                        }

                        if ((!$old_expiry_date && $expiry_date) || ($needToUpdateExpiredDate && $expiry_date)) {
                            $expDate = $expiry_date->format('Y-m-d H:i:s');
                            update_post_meta($payment->get_listing_id(), 'expiry_date', $expDate);
                        }

                    } else {
                        $current_date->add(new \DateInterval("P{$visible}D"));
                        $expDate = $current_date->format('Y-m-d H:i:s');
                        update_post_meta($payment->get_listing_id(), 'expiry_date', $expDate);
                    }

                    // Check if post expired , then turn it to published
                    if ("rtcl-expired" === $listing->get_status()) {
                        wp_update_post(array(
                            'ID'          => $listing->get_id(),
                            'post_status' => 'publish'
                        ));
                    }

                    // Todo : need to add top feature
                    update_post_meta($payment->get_id(), '_applied', 1);

                    // Hook for developers
                    do_action('rtcl_payment_completed', $post->ID);
                }
            }

            // send emails
            if (Functions::get_option_item('rtcl_email_settings', 'notify_users', 'order_completed', 'multi_checkbox')) {
                rtcl()->mailer()->emails['Order_Completed_Email_To_Customer']->trigger($payment->get_id(), $payment);
            }

            if (Functions::get_option_item('rtcl_email_settings', 'notify_admin', 'order_completed', 'multi_checkbox')) {
                rtcl()->mailer()->emails['Order_Completed_Email_To_Admin']->trigger($payment->get_id(), $payment);
            }
        }

    }

}